/* compound.h -- Compound object interface.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef PWG_COMPOUND_H
#define PWG_COMPOUND_H

#include "pwg.h"

bool chr_p (pwg_t c);
integer_t unchr (pwg_t c);
integer_t chr_digit_to_int (pwg_t c);
pwg_t pwg_digit_to_sint (pwg_t c);
pwg_t chr (integer_t n);

pwg_t pwg_chr_to_sint (pwg_t c);
pwg_t pwg_sint_to_chr (pwg_t i);
pwg_t pwg_chr_p (pwg_t o);

pwg_t pwg_cons (pwg_t car, pwg_t cdr);
void cons_gc_mark (pwg_t o);
pwg_t pwg_cons_p (pwg_t o);
pwg_t pwg_car (pwg_t arg);
pwg_t pwg_cdr (pwg_t arg);
pwg_t pwg_set_car (pwg_t arg, pwg_t x);
pwg_t pwg_set_cdr (pwg_t arg, pwg_t x);

pwg_t pwg_size (pwg_t lst);
pwg_t pwg_reverse (pwg_t lst);
pwg_t pwg_reverse_x (pwg_t lst);
pwg_t pwg_shallow_copy (pwg_t lst);

#define LIST_FOR_EACH(lst, ptr) \
  for (pwg_t ptr = lst; !eq_p (ptr, PWG_NIL); ptr = pwg_cdr (ptr))

pwg_t pwg_ary_new (pwg_t lst);
void ary_gc_mark (pwg_t o);
pwg_t pwg_ary_p (pwg_t o);
pwg_t pwg_ary_size (pwg_t arg);
pwg_t pwg_ary_item (pwg_t a, pwg_t n);
pwg_t pwg_ary_set_item (pwg_t a, pwg_t n, pwg_t x);

pwg_t str_new (const char *s);
pwg_t pwg_version_str (void);
char *unstr (pwg_t str) PWG_GCC_MALLOC;
bool str_eql (pwg_t str, const char *cstr);
pwg_t pwg_str_p (pwg_t o);
pwg_t pwg_str_copy (pwg_t s);
pwg_t pwg_str_append (pwg_t s, pwg_t t);
pwg_t pwg_str_ref (pwg_t s, pwg_t p); /* Strs are immutable.  */
pwg_t pwg_str_slice (pwg_t s, pwg_t start, pwg_t len);
pwg_t pwg_str_size (pwg_t arg);
pwg_t pwg_str_eql_p (pwg_t s, pwg_t t);
pwg_t pwg_str_sym_eql_p (pwg_t str, pwg_t sym);
pwg_t pwg_str_hash (pwg_t s);
pwg_t pwg_list_to_str (pwg_t lst);
pwg_t pwg_str_to_list (pwg_t s);
pwg_t pwg_str_to_sint (pwg_t s);
pwg_t pwg_str_to_float (pwg_t s);

#define STR_APP3(a, b, c) (pwg_str_append (a, pwg_str_append (b, c)))

pwg_t pwg_closure_new (pwg_t obj, pwg_t env);
void closure_gc_mark (pwg_t o);
pwg_t pwg_closure_p (pwg_t o);
pwg_t pwg_closure_data (pwg_t o);
pwg_t pwg_closure_env (pwg_t o);

pwg_t pwg_proc_new (pwg_t code, pwg_t args, pwg_t extra_args,
		    pwg_t re_p, pwg_t open_space_p);
void proc_gc_mark (pwg_t o);
pwg_t pwg_proc_p (pwg_t o);
pwg_t pwg_proc_code (pwg_t o);
pwg_t pwg_proc_args (pwg_t o);
pwg_t pwg_proc_extra_args (pwg_t o);
pwg_t pwg_proc_re_p (pwg_t o);
pwg_t pwg_proc_open_space_p (pwg_t o);

pwg_type_t type_new (void);
pwg_t pwg_type_new (void);
pwg_t pwg_wrap (pwg_t type, pwg_t obj);
pwg_t pwg_unwrap (pwg_t obj);

pwg_t pwg_quotor_new (pwg_t o);
pwg_t pwg_settor_new (pwg_t o);
pwg_t pwg_pushor_new (pwg_t o);

pwg_t pwg_copy (pwg_t o);

void setup_compound (void);
void init_compound (void);
void finish_compound (void);

#endif /* PWG_COMPOUND_H */
