/* enter.c -- Entrance point.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "pwg.h"

#include "stream.h"
#include "eval.h"
#include "compound.h"
#include "symkey.h"
#include "mem.h"

void
setup_enter (void)
{}

static void
posix_error (char *msg)
{
  printf ("%s: %s\n", msg, strerror (errno));
  exit (1);
}

void
init_enter (void)
{
  extern int global_argc;
  extern char **global_argv;

  pwg_module_load (sym ("default"), pwg_cons (sym ("core"), PWG_NIL));
  pwg_module_load (sym ("deps"), pwg_common_modules ());

  if (global_argc == 1)
    {
      puts ("Conseptizer " PWG_VERSION);
      repl ();
      return;
    }

  bool works_p;
  char *project = unstr (pwg_str_append (str_new (global_argv[1]),
					 str_new ("/project.cnz")));
  works_p = (access (project, R_OK) == 0);
  mem_free (project);
  if (!works_p)
    posix_error ("could not open project file");
  // FIXME: should not chdir.
  works_p = (chdir (global_argv[1]) == 0);
  if (!works_p)
    posix_error ("could not enter project directory");

  pwg_module_load (sym ("project"),
		   pwg_cons (sym ("core"), pwg_cons (sym ("deps"), PWG_NIL)));
}

void
finish_enter (void)
{}
