/* eval.h -- Evaluator interface.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef PWG_EVAL_H
#define PWG_EVAL_H

#include <setjmp.h>

#include "pwg.h"

typedef void (*psub_fn_t) (void);

pwg_t psub_new (psub_fn_t fn, integer_t args);
psub_fn_t psub_fn (pwg_t obj);
integer_t psub_args (pwg_t obj);
void psub_call (pwg_t obj);
pwg_t pwg_psub_copy (pwg_t obj);

void def_core_psub_internal (const char *name, psub_fn_t fn,
			      integer_t args);

#define def_core_psub(n, fn, args) \
  def_core_psub_internal (n, (psub_fn_t) fn, args)

void mark_modules (void);
void mark_stack (void);

pwg_t pwg_data_stack (void);
void push_data (pwg_t obj);
pwg_t pop_data (void);

#define call(x) (((void) (x)), pop_data ())

typedef void (*order_iter_t) (pwg_t obj, void *hook);
void in_order (pwg_t lst, order_iter_t fn, void *hook);
pwg_t pwg_in_order (pwg_t proc, pwg_t lst);

void eval (pwg_t lst, pwg_t env);
void repl ();

pwg_t pwg_common_modules (void);
pwg_t pwg_module_load (pwg_t name, pwg_t used_modules);
pwg_t pwg_core_module (void);

pwg_t pwg_call (pwg_t subr);
pwg_t pwg_if (pwg_t cond, pwg_t then_part, pwg_t else_part);
pwg_t pwg_eq_p (pwg_t a, pwg_t b);
pwg_t pwg_nil (void);
pwg_t pwg_eof (void);

void setup_eval (void);
void init_eval (void);
void finish_eval (void);

struct exc
{
  jmp_buf jb;
  struct exc *next;
};

void exc_register_internal (struct exc *e);
void exc_unregister_internal (void);

#define EXC_TRY do {				\
  struct exc exc_try_e;				\
  exc_register_internal (&exc_try_e);		\
  switch (setjmp (exc_try_e.jb))		\
    {						\
    case 0:

#define EXC_CASE_INTERNAL(var, num) 		\
    case num: var = num; break;

#define EXC_USER 1
#define EXC_ERR 2

#define EXC_ENSURE(var) var = 0; break;		\
      EXC_CASE_INTERNAL (var, EXC_USER);	\
      EXC_CASE_INTERNAL (var, EXC_ERR);		\
    }						\
    exc_unregister_internal ();

#define EXC_END } while (0)
#define EXC_FWD(var) if (var != 0) exc_raise (var); EXC_END

void exc_store (pwg_t val);
void exc_raise (int num);
void exc (const char *type, pwg_t msg);

pwg_t pwg_exc_forward (void);
pwg_t pwg_exc_last (void);
pwg_t pwg_exc_call_stack (void);
pwg_t pwg_exc_raise (pwg_t val);
pwg_t pwg_exc_try (pwg_t try, pwg_t ensure, pwg_t except);
pwg_t pwg_exc_hash (pwg_t sym, pwg_t msg);

#endif /* PWG_EVAL_H */
