#!/bin/sh
# gen-make.sh -- Create the Makefile.
# Copyright (C) 2014 Wolfgang Jaehrling
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# Usage:
# call this with a directory name as an argument or without.

##### User configuration. ########################

CC="gcc"
CC_FLAGS="-g"
#CC_FLAGS="-g -pg"
#CC_FLAGS="-O2"
COMPILE_FLAGS="-DPWG_DEBUG=1 -Wall -W -Wno-unused -pedantic -std=gnu99"
#COMPILE_FLAGS="-DPWG_FAST=1 -Wall -W -Wno-unused -std=gnu99"
LINK_FLAGS=""
#LINK_LIBS="-lncurses"

##### Hacker configuration. ######################

COMPILE="$CC $CC_FLAGS -c $COMPILE_FLAGS -I. -o \$@ \$<"
LINK="$CC $CC_FLAGS $LINK_FLAGS -o \$@ \$+ $LINK_LIBS"

# Plain data files.
DATA="modules"
# Scripts that generate other files.
SCRIPTS="gen-make.sh gen-init.sh"
# Files generated by scripts.
GENERATED="Makefile init.c"

##### Script code starts here. ###################

dir="${1:-.}"
modules=`cat $dir/modules`
exec 3>"$dir/Makefile"
out () { echo >&3 "$@"; }

out -n "# Generated by gen-make.sh
all: $dir/Makefile $dir/conseptizer

init.o: init.c
	$COMPILE

init.c: $dir/modules gen-init.sh
	./gen-init.sh $dir/modules

$dir/conseptizer: init.o"
for m in $modules
do
  out -n " $m.o"
done
out "
	$LINK

clean:
	rm -f *.o lists-on-a-stack core

maintainer-clean: clean
	rm -f $GENERATED
"
for m in $modules
do
  out -n "$m.o: $m.c pwg.h"
  for n in $modules
  do
    REGEX="^#include \"$n.h\"\$"
    if test -n "`grep "$REGEX" $m.c`"
    then
      out -n " $n.h"
    fi
  done
  out "
	$COMPILE
"
done

# Trigger rebuilding of init code if module list has changed.
rm -f init.o init.c
