/* hash.h -- Hash table interface.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef PWG_HASH_H
#define PWG_HASH_H

#include "pwg.h"

typedef struct hash *hash_t;
typedef void (*hash_iter_t) (void *, natural_t, natural_t);
typedef void (*hash_iter_pwg_t) (void *, pwg_t, pwg_t);

hash_t hash_new (unsigned initsize) PWG_GCC_MALLOC;
void hash_free (hash_t ht);
void hash_set (hash_t ht, natural_t key, natural_t val);
natural_t hash_val (hash_t ht, natural_t key);
void hash_remove (hash_t ht, natural_t key);
void hash_iterate (hash_t ht, hash_iter_t fn, void *hook);

pwg_t pwg_hash_new (pwg_t size);
pwg_t pwg_hash_copy (pwg_t o);
pwg_t pwg_hash_p (pwg_t h);
pwg_t pwg_hash_default (pwg_t h);
pwg_t pwg_hash_set_default (pwg_t h, pwg_t v);
pwg_t pwg_hash_has_default_p (pwg_t h);
pwg_t pwg_hash_set_item (pwg_t h, pwg_t k, pwg_t v);
pwg_t pwg_hash_item (pwg_t h, pwg_t k);
pwg_t pwg_hash_remove (pwg_t hash, pwg_t key);
pwg_t pwg_hash_iterate (pwg_t subr, pwg_t hash);

void hash_iterate_pwg (pwg_t hash, hash_iter_pwg_t fn, void *hook);

void hash_gc_mark (pwg_t ht);

void setup_hash (void);
void init_hash (void);
void finish_hash (void);

#endif /* PWG_HASH_H */
