/* mem.h -- Memory management utillity interface.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef PWG_MEM_H
#define PWG_MEM_H

#include <stddef.h>

#include "pwg.h"

void mem_info (void);

void *mem_alloc (size_t n, const char *desc) PWG_GCC_MALLOC;
void mem_free (void *data);

pwg_t gc_alloc (pwg_type_t t, size_t n);
void gc_mark (pwg_t o);
pwg_t pwg_gc (void);

void mem_setmark (pwg_t o);
void mem_unsetmark (pwg_t o);

pwg_type_t mem_ref_type (pwg_t o);
bool mem_is_type_p (pwg_t o, pwg_type_t type);
void mem_set_type (pwg_t o, pwg_type_t t);
void *mem_ref_obj (pwg_t o, pwg_type_t type);
void mem_set_obj (pwg_t o, pwg_type_t type, void *x);

void setup_mem (void);
void init_mem (void);
void finish_mem (void);

/* Define a pwg type for some C type.  NAME_AND_ARGS is the
   constructor name and its formal argument list, GET is the
   getter-name, CTYPE is the C-level type, PWGTYPE is the pwg
   type name, ONEW is the expression used for creating a new
   object. */
#define DEF_TYPE(name_n_args, get, ctype, pwgtype, onew)	\
static ctype							\
get (pwg_t o)							\
{								\
  return *((ctype *) mem_ref_obj (o, pwgtype));			\
}								\
DEF_CONSTRUCTOR (name_n_args, ctype, pwgtype, onew)

#define DEF_CONSTRUCTOR(name_n_args, ctype, pwgtype, onew)	\
pwg_t								\
name_n_args							\
{								\
  ctype x = onew;						\
  pwg_t result = gc_alloc (pwgtype, sizeof (void *));	\
  mem_set_obj (result, pwgtype, x);				\
  return result;						\
}								\
extern void dummy (void)

#endif /* PWG_NUM_H */
