/* stream.h -- Input/Output stream interface.
 * Copyright (C) 2014 Wolfgang Jaehrling
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef PWG_STREAM_H
#define PWG_STREAM_H

#include "pwg.h"
#include "hash.h"

/* A function that returns the next character.  */
typedef pwg_t (*stream_readfn_t) (void *);
/* A function that accepts a character to be written.  */
typedef void (*stream_writefn_t) (void *, pwg_t);
/* A function that marks internally used objects for the GC.  */
typedef void (*stream_markfn_t) (void *);
/* Function to destroy a stream.  */
typedef void (*stream_destructfn_t) (void *);

typedef enum { STREAM_STR_IN, STREAM_STR_OUT,
	       STREAM_FILE_IN, STREAM_FILE_OUT } stream_type_t;
typedef struct stream *stream_t;

void stream_gc_mark (pwg_t o);
void mark_stdstream (void);

pwg_t stream_new (void *hook, pwg_t name, stream_type_t type,
		  stream_readfn_t readfn,
		  stream_writefn_t writefn,
		  stream_markfn_t markfn,
		  stream_destructfn_t destructfn);
void stream_free (stream_t s);

pwg_t stream_read (stream_t s);
void stream_ungetc (stream_t s, pwg_t c);
pwg_t stream_lookahead (stream_t s);
void stream_write (stream_t s, pwg_t c);
void stream_write_str (stream_t s, const char *p);

void stream_pwg_set_line (pwg_t obj, int n);

pwg_t pwg_stream_read (pwg_t obj);
pwg_t pwg_stream_ungetc (pwg_t obj, pwg_t c);
pwg_t pwg_stream_lookahead (pwg_t obj);
pwg_t pwg_stream_write (pwg_t obj, pwg_t c);
pwg_t pwg_stream_write_str (pwg_t stream, pwg_t str);
pwg_t pwg_stream_name (pwg_t obj);
pwg_t pwg_stream_input_line (pwg_t obj);
pwg_t pwg_stream_read_line (pwg_t stream);

pwg_t pwg_str_src_new (pwg_t str);
pwg_t pwg_str_dst_new (void);
pwg_t pwg_str_dst_to_str (pwg_t s);

pwg_t pwg_file_src_new (pwg_t filename);
pwg_t pwg_file_src_close (pwg_t s);
pwg_t pwg_file_dst_new (pwg_t filename, pwg_t append_p);
pwg_t pwg_file_dst_close (pwg_t s);

stream_t stream_stdout (void);
stream_t stream_stderr (void);
stream_t stream_stdin (void);
pwg_t pwg_stream_stdout (void);
pwg_t pwg_stream_stderr (void);
pwg_t pwg_stream_stdin (void);



pwg_t in (stream_t src);
pwg_t pwg_stream_in (pwg_t stream);
pwg_t pwg_stream_out (pwg_t stream, pwg_t obj);

void setup_stream (void);
void init_stream (void);
void finish_stream (void);

#endif /* PWG_STREAM_H */
